/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 * Date:	10-Feb-2018
 */

#ifndef __GEOMETRY_TORUSBODY_H
#define __GEOMETRY_TORUSBODY_H

#include "gbody.h"

// ================================ GTorusBody =================================
class GTorusBody : public GBody {
private:
	double	ra, rb, rc;
public:
	GTorusBody(const std::string& aname, const Type atype)
		: GBody(aname, atype), ra(0.0), rb(0.0), rc(0.0) { }

	void	set(const double x, const double y, const double z,
		    const double a, const double b, const double c);

	virtual	void	set(double *);
	virtual	int	get(double *) const;
	virtual int	nwhats()	const	{ return 6; }
	virtual	int	nodes()		const	{ return 3; }
//	virtual	int	closest(const Point& r, const double d, const Vector& w) const;
//	virtual void	move(int item, const Point& r, const Vector& w);

// FIXME
//	virtual	double	distance(const double x, const double y, const double z) const;

	// Dimension labels and show flags
	// @return NULL not to show anything otherwise label
//	virtual	char*	showX()	const { return "Rx"; }
//	virtual char*	showY()	const { return "Ry"; }
//	virtual char*	showZ()	const { return "H"; }

protected:
	virtual	void	createQuads();
	virtual void	createMesh();
	virtual	void	updateBBox() const;
#ifdef OBBOX
	virtual	void	updateOBB(bool in=false) const;
#endif
}; // GTorusBody

#endif
