/**
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 * Date:	15-Jul-2020
 */

#include "clipregion.h"

// ================================ ClipRegion =================================
/** line */
bool ClipRegion::line(int* x1, int* y1, int* x2, int* y2) const
{
	OutCodeUnion ocu1, ocu2;
	bool   in;
	bool   out;
	bool   swap = false;

	/* Initialize 4-bit codes */
	ocu1.outcodes = 0;
	ocu1.ocs.code0 = (*x1 < left);
	ocu1.ocs.code1 = (*y1 < top);
	ocu1.ocs.code2 = (*x1 > right);
	ocu1.ocs.code3 = (*y1 > bottom);

	ocu2.outcodes = 0;
	ocu2.ocs.code0 = (*x2 < left);
	ocu2.ocs.code1 = (*y2 < top);
	ocu2.ocs.code2 = (*x2 > right);
	ocu2.ocs.code3 = (*y2 > bottom);

	in  = ((ocu1.outcodes | ocu2.outcodes) == 0);
	out = ((ocu1.outcodes & ocu2.outcodes) != 0);

	while (!out && !in) {
		if (ocu1.outcodes==0) {		/* Swap endpoints if necessary so */
			Swap(*x1,*x2);
			Swap(*y1,*y2);
			Swap(ocu1.outcodes,ocu2.outcodes);
		}

		if (ocu1.ocs.code0) {		/* Clip left */
			*y1 += (int)((long)((*y2-*y1) * (left-*x1))/(*x2-*x1));
			*x1 = left;
		}
		else
		if (ocu1.ocs.code1) {		/* Clip above */
			*x1 += (int)((long)((*x2-*x1) * (top-*y1))/(*y2-*y1));
			*y1 = top;
		}
		else
		if (ocu1.ocs.code2) {		/* Clip right */
			*y1 += (int)((long)((*y2-*y1) * (right-*x1))/(*x2-*x1));
			*x1 = right;
		}
		else
		if (ocu1.ocs.code3) {		/* Clip below */
			*x1 += (int)((long)((*x2-*x1) * (bottom-*y1))/(*y2-*y1));
			*y1 = bottom;
		}

		/* update for (*x1,*y1) */
		ocu1.outcodes = 0;
		ocu1.ocs.code0 = (*x1 < left);
		ocu1.ocs.code1 = (*y1 < top);
		ocu1.ocs.code2 = (*x1 > right);
		ocu1.ocs.code3 = (*y1 > bottom);

		in  = ((ocu1.outcodes | ocu2.outcodes) == 0); /* update      */
		out = ((ocu1.outcodes & ocu2.outcodes) != 0); /* 4-bit codes */
	}
	if (swap) {
		// Swap back to original order
		Swap(*x1,*x2);
		Swap(*y1,*y2);
	}
	return in;
}  /* line */

