/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 * Date:	17-Jan-2019
 */

#ifndef __MATH_SURFACE_H
#define __MATH_SURFACE_H

#include "point.h"

class Matrix4;
class BBox;
#ifdef OBBOX
class OBBox;
#endif

// ================================== Surface ==================================
/** abstract class for defining a 3D surface
 * Plane and Quad should inherit for it
 */
class Surface {
public:
	Surface()  {}

virtual	int	degree() const { return 0; }

	/** initialize if necessary some variables */
virtual	void	init() {}

	/** evaluate gradient at point p */
virtual	Vector	grad(const Point& p) const = 0;

	/** @return minimum safe distance from point p.
	 * Note: if >0 then the point is outside
	 *          <0 then the point is inside
	 */
virtual	double	distance(const Point& p) const = 0;

	/** @return intersection distance with line p+v*t */
virtual	int	distance(const Point& p, const Vector v, double t[]) const = 0;

//virtual	void	translate(const Vector& v) {}

//	void	matrix3(Matrix3 *m) const = 0;
//	void	matrix(Matrix4 *m) const = 0;
//virtual	void	transform(const Matrix4& m) = 0;
//	void	negate() = 0;

virtual	void	normalize() {}

	/** @return true if point p lies on plane */
virtual	bool	on(const Point& p, const double eps)	const
			{ return Eq0(distance(p), eps); }

	/** @return true if point p lies on positive side */
virtual	bool	onPositive(const Point& p) const { return distance(p) >= 0.0; }

	// bounding box
//virtual	BBox	bbox() const = 0;
#ifdef OBBOX
//virtual	OBBox	obbox() const = 0;
#endif
}; // class Surface

#endif
