/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 * Date:	04-Feb-2002
 */

#include "timer.h"

#include <unistd.h>
#include <sys/time.h>

/** @return time from unix epoch in seconds as double */
double btime()
{
	struct timeval tv;
	struct timezone tz;
	gettimeofday(&tv,&tz);
	return (double)tv.tv_sec + (double)tv.tv_usec/1000000.0;
} // btime

// =================================== Timer ===================================
/* ------------ start -------------- */
void Timer::start()
{
	struct timeval tv;
	struct timezone tz;

	gettimeofday(&tv,&tz);
	startRealTime = (double)tv.tv_sec + (double)tv.tv_usec/1000000.0;
	startUserTime = (double)clock() / (double)CLOCKS_PER_SEC;

	active = true;
	_laps  = 0;
} // start

/* ------------ lap -------------- */
void Timer::lap()
{
	struct timeval tv;
	struct timezone tz;

	gettimeofday(&tv,&tz);
	stopRealTime = (double)tv.tv_sec + (double)tv.tv_usec/1000000.0;
	stopUserTime = (double)clock() / (double)CLOCKS_PER_SEC;
	_laps++;
} // lap

/* ------------ stop -------------- */
void Timer::stop()
{
	struct timeval tv;
	struct timezone tz;

	gettimeofday(&tv,&tz);
	stopRealTime = (double)tv.tv_sec + (double)tv.tv_usec/1000000.0;
	stopUserTime = (double)clock() / (double)CLOCKS_PER_SEC;

	active = false;
} // stop

/* ------------ resume -------------- */
void Timer::resume()
{
	struct timeval tv;
	struct timezone tz;

	double rt = realTime();		// remember real and user time
	double ut = userTime();

	gettimeofday(&tv,&tz);
	startRealTime = (double)tv.tv_sec + (double)tv.tv_usec/1000000.0;
	startUserTime = (double)clock() / (double)CLOCKS_PER_SEC;

	startRealTime -= rt;		// remove it from the start time
	startUserTime -= ut;

	active = true;
} // resume

/* ------------ delayRealTime -------------- */
void Timer::delayRealTime(double t)
{
	struct timeval  tv;
	struct timezone tz;
	double ela;

	start();
	do {
		gettimeofday(&tv,&tz);
		ela = (double)tv.tv_sec + (double)tv.tv_usec/1000000.0;
	} while (ela<t);
	active = false;
} // delayRealTime

/* ------------ delayUserTime -------------- */
void Timer::delayUserTime(double t)
{
	struct timeval  tv;
	struct timezone tz;
	double ela;

	start();
	do {
		gettimeofday(&tv,&tz);
		ela = (double)clock() / (double)CLOCKS_PER_SEC;
	} while (ela<t);
	active = false;
} // delayUserTime
