/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 */

#include "layer.h"

#include "viewer.h"
#include "math/point.h"
#include "graph/zpainter.h"

// XXX XXX XXX
// FIXME why I cannot inline them?
// maybe make them as MACROS
// XXX XXX XXX

/** Layer */
Layer::Layer(const Geometry& g, GeometryKernel& k, GeometryViewer& v) :
	geometry(g),
	kernel(k),
	viewer(v),
	show(0)
{
} // Layer

/** view */
ViewPort& Layer::view() const
{
	return kernel.view;
} // view

/** engine */
GeometryEngine* Layer::engine() const
{
	return kernel.engine();
} // engine

#ifdef THREAD
/** engine */
GeometryEngine* Layer::engine(int i) const
{
	return kernel.engine(i);
} // engine
#endif

/** stop */
bool Layer::stop() const
{
	return viewer.stop();
} // stop

/** OLD 3D line fake coloring
 * draw the projection of a 3D line
 */
bool Layer::draw3Dline(ZPainter& painter, const Point& a, const Point& b,
			const Color3D& color)
{
	Point ra, rb;

	if (view().clipLine3D(a, b, &ra, &rb)) {
		dword col;
		if (ra.z>SMALL3D && rb.z>SMALL3D) {
			// FIXME normally should be broken into two parts >0 and <0
			col = color.bright;
		} else
		if (ra.z<-SMALL3D && rb.z<-SMALL3D)
			col = color.color;
		else
			col = color.dark;

		int x1 = view().u2i(ra.x);
		int y1 = view().v2j(ra.y);
		int x2 = view().u2i(rb.x);
		int y2 = view().v2j(rb.y);

		if (painter.lineAntialias(x1,y1, x2,y2, col))
			return true;
	}
	return false;
} // draw3Dline

/** NEW 3D line using the z-buffer
 * draw the projection of a 3D line
 */
bool Layer::draw3Dline(ZPainter& painter, const Point& a, const Point& b,
			dword color, int alpha)
{
	Point ra, rb;
	if (view().clipLine3D(a, b, &ra, &rb)) {
		int x1 = view().u2i(ra.x);
		int y1 = view().v2j(ra.y);
		int x2 = view().u2i(rb.x);
		int y2 = view().v2j(rb.y);
		//if (painter.lineAntialias(x1,y1, x2,y2, color))
		if (painter.zline(x1,y1,-ra.z, x2,y2,-rb.z, color, alpha))
			return true;
	}
	return false;
} // draw3Dline

/** drawBBox */
void Layer::drawBBox(ZPainter& painter, const BBox& bbox, const Color3D& color)
{
	if (!bbox.isValid()) return;

	for (int i=0; i<bbox.edges(); i++) {
		int a, b;
		bbox.edge(i, &a, &b);
		draw3Dline(painter, bbox.vertex(a), bbox.vertex(b), color.color, 200);
	}
} // drawBBox

#ifdef OBBOX
/** drawOBBox */
void Layer::drawOBBox(ZPainter& painter, const OBBox& bbox, const Color3D& color)
{
	if (!bbox.isValid()) return;

	for (int i=0; i<bbox.edges(); i++) {
		int a, b;
		bbox.edge(i, &a, &b);
		draw3Dline(painter, bbox.vertex(a), bbox.vertex(b), color.color, 200);
	}
} // drawOBBox
#endif

