/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author: Vasilis.Vlachoudis@cern.ch
 * Date: 2015
 */

#ifndef __VIEWER_MGDRAW_LAYER_H
#define __VIEWER_MGDRAW_LAYER_H

#include "layer.h"
#include "graph/color.h"
#include "geometry/geo.h"
#include "data/userdump.h"

class ZPainter;

/* ============================= UserDumpLayer ============================== */
class UserDumpLayer : public Layer {
protected:
static	const int NEG_PARTICLES	= 6;			// negative particles
static	const int NPARTICLES	= 64+NEG_PARTICLES;

protected:
	UserDump	userdump;	// userdump class
	time_t		_mtime;
	std::string	_filename;	// filename

public:
	int	start;			// starting source event
	int	n;			// how many events to show
	int	alpha;			// alpha blending for background tracks

	dword	_color[NPARTICLES];	// particle color transparent = no show
	double	_emin[NPARTICLES];	// energy limits to show
	double	_emax[NPARTICLES];

public:
	UserDumpLayer(const Geometry& g, GeometryKernel& k, GeometryViewer& v);
	~UserDumpLayer();

	void	cleanup();

	/* set/get */
const	std::string&	filename()	const	{ return _filename; }
	bool	open(const char *name=NULL);
	void	reset();

	void	display(int particle)		{ _color[particle+NEG_PARTICLES] &= ~COLOR_ALPHA; }
	void	hide(int particle)		{ _color[particle+NEG_PARTICLES] |=  COLOR_ALPHA; }
	void	color(int particle, dword c)	{ _color[particle+NEG_PARTICLES] = c; }
	dword	color(int particle)	const	{
			if (particle>=-NEG_PARTICLES && particle<NPARTICLES)
				return _color[particle+NEG_PARTICLES];
			else
				return COLOR_ALPHA;
		}
	void	emin(int particle, double e)	{ _emin[particle+NEG_PARTICLES] = e; }
	double	emin(int particle) const {
			if (particle>=-NEG_PARTICLES && particle<NPARTICLES)
				return _emin[particle+NEG_PARTICLES];
			else
				return 0.0;
		}

	void	emax(int particle, double e)	{ _emax[particle+NEG_PARTICLES] = e; }
	double	emax(int particle) const {
			if (particle>=-NEG_PARTICLES && particle<NPARTICLES)
				return _emax[particle+NEG_PARTICLES];
			else
				return 1e99;
		}

	void	draw(ZPainter& painter, bool show3D);
}; // UserDumpLayer

#endif
